@@ -137,18 +137,23 @@ def _nanopb_proto_actions(source, target, env, for_signature):
         if not os.path.isabs(d): d = os.path.relpath(d, prefix)
         include_dirs += ' -I' + esc(d)
 
+    # when generating .pb.cpp sources, instead of pb.h generate .pb.hpp headers
+    source_extension = os.path.splitext(str(target[0]))[1]
+    header_extension = '.h' + source_extension[2:]
     nanopb_flags = env['NANOPBFLAGS']
     if nanopb_flags:
-      nanopb_flags = '%s:.' % nanopb_flags
+      nanopb_flags = '--source-extension=%s,--header-extension=%s,%s:.' % (source_extension, header_extension, nanopb_flags)
     else:
-      nanopb_flags = '.'
+      nanopb_flags = '--source-extension=%s,--header-extension=%s:.' % (source_extension, header_extension)
 
     return SCons.Action.CommandAction('$PROTOC $PROTOCFLAGS %s --nanopb_out=%s %s' % (include_dirs, nanopb_flags, srcfile),
                                       chdir = prefix)
 
 def _nanopb_proto_emitter(target, source, env):
     basename = os.path.splitext(str(source[0]))[0]
-    target.append(basename + '.pb.h')
+    source_extension = os.path.splitext(str(target[0]))[1]
+    header_extension = '.h' + source_extension[2:]
+    target.append(basename + '.pb' + header_extension)
 
     # This is a bit of a hack. protoc include paths work the sanest
     # when the working directory is the same as the source root directory.
@@ -167,6 +172,12 @@ _nanopb_proto_builder = SCons.Builder.Builder(
     src_suffix = '.proto',
     emitter = _nanopb_proto_emitter)
 
+_nanopb_proto_cpp_builder = SCons.Builder.Builder(
+    generator = _nanopb_proto_actions,
+    suffix = '.pb.cpp',
+    src_suffix = '.proto',
+    emitter = _nanopb_proto_emitter)
+
 def generate(env):
     '''Add Builder for nanopb protos.'''
 
@@ -181,6 +192,7 @@ def generate(env):
 
     env.SetDefault(NANOPB_PROTO_CMD = '$PROTOC $PROTOCFLAGS --nanopb_out=$NANOPBFLAGS:. $SOURCES')
     env['BUILDERS']['NanopbProto'] = _nanopb_proto_builder
+    env['BUILDERS']['NanopbProtoCpp'] = _nanopb_proto_cpp_builder
 
 def exists(env):
     return _detect_protoc(env) and _detect_protoc_opts(env)
